<?php

declare(strict_types=1);

namespace LidoNation\Adapayments\App\Services;

use Exception;
use LidoNation\Adapayments\App\Enums\CurrencyEnum;
use LidoNation\Adapayments\App\Http\Integrations\CoinApi\CoinApiConnector;
use LidoNation\Adapayments\App\Http\Integrations\CoinApi\Requests\GetQuoteRequest as CoinApiGetQuoteRequest;
use LidoNation\Adapayments\App\Http\Integrations\Coinbase\CoinbaseConnector;
use LidoNation\Adapayments\App\Http\Integrations\Coinbase\Requests\GetQuoteRequest as CoinbaseGetQuoteRequest;
use LidoNation\Adapayments\App\Http\Integrations\Coinmarketcap\CoinMarketCapConnector;
use LidoNation\Adapayments\App\Http\Integrations\Coinmarketcap\Requests\GetQuoteRequest as CoinmarketcapGetQuoteRequest;

class AdaUsdQuoteService
{
    public $rate;

    public $rateApiProviders = ['coinapi', 'coinbase', 'coinmarketcap'];

    public function __construct(protected CurrencyEnum|null $base = null, protected CurrencyEnum|null $quote = null)
    {
        if (! is_null($base) && ! is_null($quote)) {
            $this->getHttpRate($base, $quote);
        }
    }

    // gets the exchange rate from multiple apis then assigns the public property 'rate' of this class
    protected function getHttpRate(): self
    {
        $ratesArray = [];

        foreach ($this->rateApiProviders as $provider) {
                $rate = $this->{$provider}();
                if (!is_null($rate)) {
                    array_push($ratesArray, $rate);
                }
        }

        // if rates array is null throw error, no endpoint resolved, else calculate the average
        $avgRate = !empty($ratesArray) ? $this->calAvg($ratesArray) : throw new Exception("No exchange rate fetched, please check endpoints");
        $this->rate = $avgRate;

        return $this;
    }

    //calculates average rate from an array.
    protected function calAvg($rateArray)
    {
        return array_sum($rateArray) / count($rateArray);
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // The following are api endpoints functions

    // endpoint
    public function coinapi()
    {
            $connector = new CoinApiConnector();
            $request = new CoinApiGetQuoteRequest($this->base->name, $this->quote->name);

            $response = $connector->send($request);
            if ($response->ok()) {
                return $response->json()['rate'];
            } else {
                error_log("Error: coinapi endpoint did not resolve");
                return;
            }

    }

    // endpoint
    public function coinbase()
    {
            $connector = new CoinbaseConnector();
            $request = new CoinbaseGetQuoteRequest($this->base->name);

            $response = $connector->send($request);
            if ($response->ok()) {
                return $response->json()['data']['rates'][$this->quote->name];
            } else {
                error_log("Error: coinbase endpoint did not resolve");
                return;
            }

    }

    // endpoint
    public function coinmarketcap()
    {
            $quotes = null;

            // most endpoints need subscription. so we check for
            $connector = new CoinMarketCapConnector();
            $request = new CoinmarketcapGetQuoteRequest(2010);

            $response = $connector->send($request);
            if ($response->ok()) {
                $responseData = $response->json()['data'];
                if (isset($responseData)) {
                    $data = (object) ($responseData[2010] ?? null);
                    if (isset($data?->quote['USD'])) {
                        $quotes = (object) $data?->quote['USD'];
                    }
                }
    
                // if baseCurrency is USD, then return inverse price
                $inverse = $this->base->name === 'ADA' ? false : true;
    
                return $inverse ? 1 / $quotes->price : $quotes->price;
            } else {
                error_log("Error: coinmarketcap endpoint did not resolve");
                return;
            }
        
    }
}
